/*******************************************************************************
* FILE NAME: OI_routines.c
*
* DESCRIPTION:
*    Update the OI display LEDs and the dashboard status information
*
*******************************************************************************/

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"

#include "user_routines.h"
#include "clock_routines.h"
#include "utilities.h"
#include "velocity_control.h"


void Update_OI()
{
 /*---------- OI LEDs------------------------------------------------
  *------------------------------------------------------------------
  *   This section drives the "ROBOT FEEDBACK" lights on the Operator Interface.
  *   The lights are green for joystick forward and red for joystick reverse.
  *   Both red and green are on when the joystick is centered.  Use the
  *   trim tabs on the joystick to adjust the center.
  */	
  
  if (user_display_mode == OFF) /* User Mode is Off so the LED indicators are in use */
  { 
    // Driver on Joystick ports 1 & 2, Gunner on Joystick port 3, Switch box on port 4
    // PWM1 LEDs - Port 1 y axis
    // PWM2 LEDs - Port 2 y axis or Port1 x axis depending on 1 or 2-joystick mode
    // Relay1 LEDs - Port 3 y axis
    // Relay2 LEDs - Port 3 x axis

    /********* Joystick trim *********/
    /**** Display trim position of Port 1 Drive y Joystick ****/
    if (JOY_DRIVE1A > -NULLJOYSTICK) 
      Pwm1_green  = ON;    /* Turn Relay1 green LED - OFF */
    else
      Pwm1_green  = OFF;    /* Turn Relay1 green LED - OFF */

    if (JOY_DRIVE1A <  NULLJOYSTICK) 
      Pwm1_red  = ON;      /* Turn Relay1 red LED   - ON */
    else
      Pwm1_red  = OFF;      /* Turn Relay1 red LED   - ON */


    if (!JOYSELECT) // One joystick mode
    {
      /**** Display trim position of Port 1 Drive x Joystick ****/
      if (JOY_DRIVE1B > -NULLJOYSTICK) 
        Pwm2_green  = ON;    /* Turn green LED - OFF */
      else
        Pwm2_green  = OFF;    /* Turn green LED - OFF */

      if (JOY_DRIVE1B <  NULLJOYSTICK) 
        Pwm2_red  = ON;      /* Turn red LED   - ON */
      else
        Pwm2_red  = OFF;      /* Turn red LED   - ON */
    }
    else  // Two joystick mode
    {
      /**** Display trim position of Port 2 Drive y Joystick ****/
      if (JOY_DRIVE2 > -NULLJOYSTICK) 
        Pwm2_green  = ON;    /* Turn green LED - OFF */
      else
        Pwm2_green  = OFF;    /* Turn green LED - OFF */

      if (JOY_DRIVE2 <  NULLJOYSTICK) 
        Pwm2_red  = ON;      /* Turn red LED   - ON */
      else
        Pwm2_red  = OFF;      /* Turn red LED   - ON */
    }


    /**** Display trim position of Port 3 Shooter Joystick x ****/
    if (SHOOTER_INPUT_X > -NULLJOYSTICK) 
      Relay1_green  = ON;    /* Turn Relay1 green LED - OFF */
    else
      Relay1_green  = OFF;   /* Turn Relay1 green LED - OFF */
    if (SHOOTER_INPUT_X <  NULLJOYSTICK) 
      Relay1_red  = ON;      /* Turn Relay1 red LED   - ON */
    else
      Relay1_red  = OFF;     /* Turn Relay1 red LED   - ON */

//  Relay2_green & red are reserved as a target tracking indicator set in targeting.c


#ifdef  //Test Only
  // TEST ONLY - Strobe OI lights
    if (Clocksec%2==0)
    {
      Pwm1_green=Pwm2_green=1;
      Relay1_green=Relay2_green=1;
      Switch1_LED=Switch2_LED=Switch3_LED=1;

      Pwm1_red=Pwm2_red=0;
      Relay1_red=Relay2_red=0;
    }
    else
    {
      Pwm1_green=Pwm2_green=0;
      Relay1_green=Relay2_green=0;
      Switch1_LED=Switch2_LED=Switch3_LED=0;

      Pwm1_red=Pwm2_red=1;
      Relay1_red=Relay2_red=1;

    }
#endif
    
  } /* (user_display_mode = 0) (User Mode is Off) */
  
  else  /* User Mode is On - displays data in OI 4-digit display */
  {

 /*---------- OI User Mode Display-----------------------------------
  *------------------------------------------------------------------
  */
// Field Position + Autonomous Selected

//    User_Mode_byte = backup_voltage*10; /* so that decimal doesn't get truncated. */

    User_Mode_byte = AUTOMODE;  // Autonomous version selected

    // Field position
    if (rc_dig_in11 == 0) 
    {
      User_Mode_byte += 100; // Right
    }
    else if (rc_dig_in12 == 0)
    {
      User_Mode_byte += 200; // Left
    }
    else if (rc_dig_in13 == 0)
    {
//      User_Mode_byte += 0; // Center
    }
  }

  /**** Miscellaneous indicators ****/
//  Switch1_LED = OFF; // Gimbal Pan track
  Switch2_LED = UpToSpeed;
  Switch3_LED = relay8_fwd; // Compressor on or off

 /*---------- Dashboard Display --------------------------------------------------
  *------------------------------------------------------------------------------
  */

  User_Byte1  = OI_KP_POT>>1; // OI pot for Kp 
  User_Byte2  = OI_KI_POT>>1; // OI pot for Ki
  User_Byte3  = OI_KD_POT>>1; // OI pot for Kd

  User_Byte4  = 0; // Available
  User_Byte5  = 0; // Available
}
