/*******************************************************************************
* FILE NAME: user_routines.c
*
* DESCRIPTION:
*  This file contains the default mappings of inputs  
*  (like switches, joysticks, and buttons) to outputs on the RC.  
*
*******************************************************************************/

#include <stdio.h>

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"
#include "serial_ports.h"
#include "camera.h"
#include "tracking.h"
#include "terminal.h"
#include "utilities.h"
#include "clock_routines.h"
#include "gyro.h"
#include "auto_routines.h"
#include "velocity_control.h"
#include "targeting.h"

#define VARIABLE_DECLARE  /* causes certain global variables to be declared only once */
  #include "user_routines.h"
#undef VARIABLE_DECLARE

extern unsigned char camera_initialized;
extern unsigned int camera_t_packets;
extern unsigned int old_camera_t_packets;
extern unsigned int counter;
extern unsigned long number_o_loops;
unsigned char wepushedthebutton = 0;
unsigned char firsttime=TRUE;

/*******************************************************************************
* FUNCTION NAME: User_Initialization
* PURPOSE:       This routine is called first (and only once) in the Main function.  
*                You may modify and add to this function.
* CALLED FROM:   main.c
*******************************************************************************/
void User_Initialization (void)
{
  Set_Number_of_Analog_Channels(SIXTEEN_ANALOG);    /* DO NOT CHANGE! */

/* FIRST: Set up the I/O pins you want to use as digital INPUTS. */
  digital_io_01 = digital_io_02 = digital_io_03 = digital_io_04 = INPUT;
  digital_io_05 = digital_io_06 = digital_io_07 = digital_io_08 = INPUT;
  digital_io_09 = digital_io_10 = digital_io_11 = digital_io_12 = INPUT;
  digital_io_13 = digital_io_14 = digital_io_15 = digital_io_16 = INPUT;
  digital_io_17 = INPUT; digital_io_18 = INPUT;  /* Used for pneumatic pressure switch. */

/* SECOND: Set up the I/O pins you want to use as digital OUTPUTS. */
  digital_io_03 = OUTPUT; // Use as status indicator

/* THIRD: Initialize the values on the digital outputs. */
  rc_dig_out03 = 0;

/* FOURTH: Set your initial PWM values.  Neutral is 127. */
  pwm01 = pwm02 = pwm03 = pwm04 = pwm05 = pwm06 = pwm07 = pwm08 = 127;
  pwm09 = pwm10 = pwm11 = pwm12 = pwm13 = pwm14 = pwm15 = pwm16 = 127;

/* FIFTH: Set your PWM output types for PWM OUTPUTS 13-16.
  /*   Choose from these parameters for PWM 13-16 respectively:               */
  /*     IFI_PWM  - Standard IFI PWM output generated with Generate_Pwms(...) */
  /*     USER_CCP - User can use PWM pin as digital I/O or CCP pin.           */
  Setup_PWM_Output_Type(IFI_PWM,IFI_PWM,IFI_PWM,IFI_PWM);

  /* Add any other initialization code here. */

  /* Initialize relay outputs */
  relay1_fwd = 0; relay1_rev = 0;
  relay2_fwd = 0; relay2_rev = 0;
  relay3_fwd = 0; relay3_rev = 0;
  relay4_fwd = 0; relay4_rev = 0;
  relay5_fwd = 0; relay5_rev = 0;
  relay6_fwd = 0; relay6_rev = 0;
  relay7_fwd = 0; relay7_rev = 0;
  relay8_fwd = 0; relay8_rev = 0; // Compressor


  Init_Serial_Port_One();
  Init_Serial_Port_Two();

#ifdef TERMINAL_SERIAL_PORT_1    
  stdout_serial_port = SERIAL_PORT_ONE;
#endif

#ifdef TERMINAL_SERIAL_PORT_2    
  stdout_serial_port = SERIAL_PORT_TWO;
#endif

  Initialize_Clock();
  Initialize_Interrupts();
  firsttime = 1;
  Putdata(&txdata);            /* DO NOT CHANGE! */

  User_Proc_Is_Ready();         /* DO NOT CHANGE! - last line of User_Initialization */
}

/*******************************************************************************
* FUNCTION NAME: Process_Data_From_Master_uP
* PURPOSE:       Executes every 26.2ms when it gets new data from the master 
*                microprocessor.
* CALLED FROM:   main.c
*******************************************************************************/

void Process_Data_From_Master_uP(void)
{
   unsigned int swtc;  //set equal to the switch input

    // **** Velocity Control of the optimum shooter speed
    if (ShooterOn)
    {
      VelocityControl();
    }
    else
    {
      SHOOTER_MOTOR = NEUTRAL;
    }

    // **** Manage the camera
    Camera_Comm_Check();

	// This function is responsable for camera initialization 
	// and camera serial data interpretation. Once the camera
	// is initialized and starts sending tracking data, this 
	// function will continuously update the global T_Packet_Data 
	// structure with the received tracking information.
	Camera_Handler();

    // Note: 7 clock ticks per fast loop - baseline
//    printf("Slow loop, %d, fast loops/sec=,%ld",counter, number_o_loops*39);
//    number_o_loops = 0;
//    WriteTimer1(0);

#if 0
    swtc = AUTOMODE;  // Auto select switch 0-9
printf("swtc=%d, ", swtc);

  if (rc_dig_in11 == 0)
  {
//    start_position = STARTLEFT;
    printf(" ***left*** ");
  }
  else if (rc_dig_in12 == 0)
  {
//    start_position = STARTCENTER;
    printf(" ***center*** ");
  }
  else if (rc_dig_in13 == 0)
  {
//    start_position = STARTRIGHT;
    printf(" ***right*** ");
  }
  else
  {
//    start_position = STARTCENTER;  // default
    printf(" ***CENTER (default)*** ");
  }
printf("\r");
#endif

}

/*******************************************************************************
* FUNCTION NAME: Driver_Routine
* PURPOSE:       Performs the default mappings of inputs to outputs for the
*                Robot Controller.
*
* ASSIGNMENTS:
*    p1_sw_trig, p1_sw_top, p1_sw_aux1, p1_sw_aux2, p1_sw_wheel
*    pwm01/pwm02 - Right Drive
*    pwm03/pwm04 - Left Drive
*    pwm05 - Pan servo
*    pwm06 - Tilt servo
*******************************************************************************/
#define HOLDELAY 5  //Delay in tenths of seconds to allow slowdown to a stop
#define BALLLAUNCHDELAY 5  // 1/2 second
#define OIPOTRANGE 254
#define OIFILTER 1

void Driver_Routine(void)
{
  static char FirstTime=TRUE;
  static unsigned long LastClock=0;
  unsigned long LocalClockds;
  static unsigned char testflag=0, testflagend=0, testcounter=0;
  static int LastPanPot=0, StartPanPot=0, pantest=0;
  static char ShiftSwitchState=0; // Whatever SHIFT is equal to on startup
  static char ConveyorToggle=TRUE;
  int pan_motor=0, pan_motor2=0;
  static long Time_ball_launch=0;
  static int Last_OI_Dial = 127;
  int OI_Dial, RC_Dial;

  printf("PISTONUP=%d\r", (int)PISTONUP);

  if(FirstTime)
  {
    ShiftSwitchState = SHIFT;
    FirstTime = FALSE;
  }

//  printf("Encoders: Left=,%ld, Right=,%ld, ", Total_Right_Encoder_Count, Total_Left_Encoder_Count);
//  printf("Heading, %d, Output, %d\r",RawHeading,GyroDrive(100));

  if (!JOYSELECT) // Selectable by OI switch */
  {
    //*---------- 1 Joystick Drive ----------------------------------------------
    //*--------------------------------------------------------------------------

//     print358(("1 Joystick Drive!\r"));
//     Smooth_Joystick(&JOY_DRIVE1A);
//     Smooth_Joystick(&JOY_DRIVE1B);

     WHL_R1 = WHL_R2 = -Limit_Mix(JOY_DRIVE1A + JOY_DRIVE1B);
     WHL_L1 = WHL_L2 = -Limit_Mix(JOY_DRIVE1A - JOY_DRIVE1B);

//     if (p1_sw_trig == 1) WHL_L1 = WHL_L2 = WHL_L2*-1;

     JOY_DRIVE2 = 0;
  }
  else
  {
   //*---------- 2 Joystick Drive ----------------------------------------------
   //*--------------------------------------------------------------------------

//    print358(("2 Joystick Drive!\r"));
//    Smooth_Joystick(&JOY_DRIVE2);
//    Smooth_Joystick(&JOY_DRIVE1A);

    WHL_R1 = WHL_R2 = -JOY_DRIVE2;
    WHL_L1 = WHL_L2 = -JOY_DRIVE1A;
    JOY_DRIVE1B = 0;
//    printf("Right=%d, Left=%d\r", WHL_R1, WHL_L1);
  }

   //*---------- Shooter Pan gimbal --------------------------------------------
   //*--------------------------------------------------------------------------

  // Two ways to do this
#if 0
  // One: if by joystick power alone
  PAN_MOTOR = JOY_PAN/2;
  if(PAN_MOTOR > PAN_TOP_SPEED_RIGHT)
    PAN_MOTOR = PAN_TOP_SPEED_RIGHT;
  if(PAN_MOTOR < -PAN_TOP_SPEED_LEFT)
    PAN_MOTOR = -PAN_TOP_SPEED_LEFT;
#else

  // Two: if by dial a position
  OI_Dial = JOY_PAN * -1 + 127;//Reverse the direction to match the pan gimbal, 0=minimum
  // Keep within allowable range
  if (OI_Dial < OIPOTMIN) OI_Dial = OIPOTMIN;
  if (OI_Dial > OIPOTMAX) OI_Dial = OIPOTMAX;
  // Filter OI input
  if ((OI_Dial > (Last_OI_Dial+OIFILTER)) || (OI_Dial < (Last_OI_Dial-OIFILTER)))
  { // The OI control has moved significantly otherwise it's just flickering
    Last_OI_Dial = OI_Dial;
  }
  // Map OI dial to pan potentiometer
  RC_Dial = (long)(Last_OI_Dial - OIPOTMIN) * (PANPOTMAX - PANPOTMIN) / (OIPOTMAX - OIPOTMIN) + PANPOTMIN;  

  Pan_By_Pot(RC_Dial, &pan_motor);
//  printf("RC_Dial=%d, ", RC_Dial);
  PAN_MOTOR = pan_motor;
#endif

#if 1
    // Know our limits...
    if ((PANPOT <= PANPOTMIN) && (PAN_MOTOR > 0))
    {
      PAN_MOTOR = 0;
    }
    else if ((PANPOT >= PANPOTMAX) && (PAN_MOTOR < 0))
    {
      PAN_MOTOR = 0;
    }
#endif
//printf("BALLREADY=%d, ",BALLREADY);
//printf("PAN_MOTOR=%d, PANPOT=%d\r", PAN_MOTOR, PANPOT);


#if 0
   //*---------- Shooter Tilt gimbal -------------------------------------------
   //*--------------------------------------------------------------------------

  if (JOY_TILT < 0) 
  {
    TILT_MOTOR  = JOY_TILT/8;
  }
  else if (JOY_TILT > 0) 
  {
    TILT_MOTOR  = JOY_TILT/4;
  }
#if 0
  if ((TILTPOT <= TILTPOTMIN) && (TILT_MOTOR > 0))
  {
    TILT_MOTOR = 0;
  }
  else if ((TILTPOT >= TILTPOTMAX) && (TILT_MOTOR < 0))
  {
    TILT_MOTOR = 0;
  }
#endif

if ((TILTLIMIT==0) && (TILT_MOTOR < 0))
  {
    TILT_MOTOR = 0;
  }
//  printf("PANPOT=%d, TILTPOT=%d\r", PANPOT, TILTPOT);
#endif //Tilt

 //*---------- Hold Stationary Position --------------------------------------
 //*--------------------------------------------------------------------------
#if 0  // Not debugged
    if ((JOY_DRIVE1A == 0) & (JOY_DRIVE1B == 0) & (JOY_DRIVE2 == 0))
    {
      if (notmoving)
      {
       print358(("Holding Position!\r"));
        // Disable interrupts whenever we grab a volatile value
        INTCONbits.GIEL = 0;
        LocalClockds = Clockds;
        INTCONbits.GIEL = 1;
        
        if (LastClock == LocalClockds) // Force robot to "settle" after what may be a sudden stop
        {
          LastClock = 0;
          // Disable interrupts whenever we grab a volatile value
          // zero the shaft encoders
          INTCONbits.GIEL = 0;
          Right_Encoder = 0;
          Left_Encoder = 0;
          INTCONbits.GIEL = 1;
        }
        Hold_Position((int)FALSE, (int)0, (int)0);
      }
      else
      {
        // Disable interrupts whenever we grab a volatile value
        // zero the shaft encoders
        INTCONbits.GIEL = 0;
        Right_Encoder = 0;
        Left_Encoder = 0;
        INTCONbits.GIEL = 1;

        LastClock = LocalClockds + HOLDELAY;
        Hold_Position((int)TRUE, (int)0, (int)0);
        notmoving = TRUE;
      }
    }
    else
    {
      notmoving = FALSE;
    }  
#endif

   //*---------- Shift Gears ----------------------------------------------
   //*---------------------------------------------------------------------

    if (LOWGEAR) // Measured 4.98 fps in low gear reached in 8.4 seconds
    {
      SHIFTER = 1;
    }
    else if (HIGHGEAR)  // Measured 11.94 fps in high gear reached in 7 seconds
    {
      SHIFTER = 0;
    }
    else if (ShiftSwitchState != SHIFT)
    {
      SHIFTER = ShiftSwitchState = SHIFT;
    }

   //*---------- Camera Tracking -----------------------------------------------
   //*--------------------------------------------------------------------------

//FIRE_MODE2 = TRUE; // DEBUG*************************

    // Light the OI tracking light and suggest where the pan should go
    if(GimbalTrackPan(&pan_motor2))
    { 
       Switch1_LED = 1; // We are locked on target
//       if(FIRE_MODE2 && (pan_motor2==0))  // Automatic mode - actively track and fire when target lock
       if(FIRE_MODE2)  // Automatic mode - actively track and fire when target lock
       {
          FIRE1 = TRUE; // Give overiding permission to fire
          //printf( "FIRE (MODE2)\r");
       }
    }
    else
    {
       Switch1_LED = 0; // Not locked on target
    }
    if(FIRE_MODE1 || FIRE_MODE2)  // semi- or full auto mode - actively track the target
    {
      //printf(" FIRE MODE 1 \r");
      PAN_MOTOR = pan_motor2;
    }

   //*---------- Shooter Power ------------------------------------------
   //*-------------------------------------------------------------------

    ShooterOn = !SHOOTER_POWER;
    if (!ShooterOn) UpToSpeed = FALSE;


   //*---------- Fire Away ----------------------------------------------
   //*-------------------------------------------------------------------

    if (FIRE1 || FIRE2)
    {
      if(BALLREADY && UpToSpeed)
//      if(!BALLREADY)
      {
        if (!BALLLAUNCH)
        {
          BALLLAUNCH = 1;
          Time_ball_launch = Clockds + BALLLAUNCHDELAY;
        }
      }
    }
    
    if (!PISTONUP || (Time_ball_launch < Clockds))
    {
      BALLLAUNCH = 0;
    }
    
//    printf("Port 3x=%d\r", p3_x); // pot is 254 to 0

   //*---------- Conveyor ------------------------------------------------------
   //*--------------------------------------------------------------------------
    if ((CONVEYOR1 || CONVEYOR2) && ConveyorToggle)
    {
        ConveyorToggle = FALSE;
        if (HARVESTER_MOTOR == 0)
        {
          HARVESTER_MOTOR = -127;
        }
        else
        {
          HARVESTER_MOTOR = 0;
        }
    }
    if (!CONVEYOR1 && !CONVEYOR2)
    {
      ConveyorToggle = TRUE;
    }
printf("PAN_MOTOR=%d, PANPOT=%d\r", PAN_MOTOR, PANPOT);
} /* END Default_Routine(); */


/******************************************************************************/
/******************************************************************************/
/******************************************************************************/

/*******************************************************************************
* FUNCTION NAME: Pan_By_Pot
* PURPOSE:       Determine motor power to move pan to a position.
*                Treat one direction as separate and distinct from the opposite direction
*      position - pot value to match. Degress from straight would make better sense.
*      pan_motor - suggested pan motor value to correct the error
*******************************************************************************/
#define KP_R 4/10  // Basic proportional adjustment (was 8/10)
#define KI_R 0/100  // (was 0/100)
#define KD_R 10/10  //  (was 0/10)

#define KP_L 4/10  // Basic proportional adjustment (was 10/10)
#define KI_L 0/100  // (was 0/100)
#define KD_L 10/10  // (was 10/10)

#define IMAX 300  // 300 max = 127*KI
#define PANDEADBAND 10

void Pan_By_Pot(int desired_position, int *pan_motor)
{
  int error, current_position, temp=1;
  static int last_position, last_desired_position=0, last_error = 0, i=0, d=0;
  static char crossover_flag = FALSE;
//  int KP_R, KI_R, KD_R, KP_L, KI_L, KD_L;

  current_position = PANPOT;

  // Proportional
  error = (current_position - desired_position) * PAN_POT_DIRECTION;

  // Derivative
  d = last_error - error;
  last_error = error;

  // Integral
  // Zero the accumulated error if we pass the target point
  if (error > 0 && crossover_flag)
  {
    i=0;
    crossover_flag = FALSE;
  }
  else if (error < 0 && !crossover_flag)
  {
    i=0;
    crossover_flag = TRUE;
  }

  if (last_desired_position != desired_position)
  {
    i=0;
    d=0;
    last_desired_position = desired_position;
  }
    if (current_position == last_position)
    { // We are not moving, so pick up the tempo just enough to get going
      if (i == 0) 
      {
          temp = PAN_MIN_SPEED_i; // Start i just below movement speed for quicker response
      }
      else
        temp = 1;

      if (current_position < desired_position)
      {
        i -= temp;
      }
      else
      {
        i += temp;
      }
      if (i >  IMAX) i =  IMAX;
      if (i < -IMAX) i = -IMAX;
    }
    last_position = current_position;

  if ((last_error < 0) && (error > 0) || (last_error > 0) && (error < 0))
  { // we crossed over
    i = 0;
  }


  //------------------------

  if ((error < PANDEADBAND) && (error > -PANDEADBAND))  // Are we there yet?
  {
    *pan_motor = 0; // We are there - Stop, halt, pause, wait
    i = 0;
    last_error = 0;
  }
  else  // Not there yet...
  {
    // Test for commanded direction
    if (error > 0)
    {
      *pan_motor = (error * KP_L) + (i * KI_L) - (d * KD_L);
      if (*pan_motor > PAN_TOP_SPEED_LEFT) *pan_motor = PAN_TOP_SPEED_LEFT;
    }
    else
    {
      *pan_motor = (error * KP_R) + (i * KI_R) - (d * KD_R);
      if (*pan_motor < -PAN_TOP_SPEED_RIGHT) *pan_motor = -PAN_TOP_SPEED_RIGHT;
    }

    *pan_motor *= PAN_MOTOR_DIRECTION;

    // Know our limits...
    if ((PANPOT <= PANPOTMIN) && (*pan_motor > 0))
    {
      *pan_motor = 0;
    }
    else if ((PANPOT >= PANPOTMAX) && (*pan_motor < 0))
    {
      *pan_motor = 0;
    }
//    printf("Drv: desired_pos=%d, current_pos=%d, pan_motor=%d, i=%d, d=%d, temp=%d\r", 
//                        desired_position, current_position,*pan_motor, i, d, temp);
  }
}

