/*******************************************************************************
* FILE NAME: user_routines.h
*
* DESCRIPTION: 
*  This is the include file which corresponds to user_routines.c and
*  user_routines_fast.c
*  It contains some aliases and function prototypes used in those files.
*
* USAGE:
*  If you add your own routines to those files, this is a good place to add
*  your custom macros (aliases), type definitions, and function prototypes.
*******************************************************************************/

#ifndef __user_program_h_
#define __user_program_h_


#ifdef VARIABLE_DECLARE
  volatile signed long Right_Encoder=0,Left_Encoder=0;
  volatile signed long Total_Right_Encoder_Count=0,Total_Left_Encoder_Count=0;
  char AutoFirstTime=TRUE;  // Identifies when Autonomous mode first starts
  char DriverFirstTime=FALSE;  // Identifies when the human driver takes control
  char notmoving=FALSE;
#else
  extern volatile signed long Right_Encoder,Left_Encoder;
  extern volatile signed long Total_Right_Encoder_Count,Total_Left_Encoder_Count;
  extern char AutoFirstTime;
  extern char DriverFirstTime;
  extern char notmoving;
#endif


/*******************************************************************************
                            MACRO DECLARATIONS
*******************************************************************************/

/* Used in limit switch routines in user_routines.c */
#define OPEN        1     /* Limit switch is open (input is floating high). */
#define CLOSED      0     /* Limit switch is closed (input connected to ground). */
#define ON                1
#define OFF               0
#define PACKET_MISS_THRESH 8
#define MAX_SPEED   127

// Relay outputs
#define SHIFTER     relay6_fwd
#define BALLLAUNCH  relay7_fwd
#define COMPRESSOR  relay8_fwd

// PWMs
#define WHL_L1	        ipwm01
#define WHL_L2	        ipwm02
#define WHL_R1	        ipwm03
#define WHL_R2	        ipwm04
#define SHOOTER_MOTOR   ipwm05
#define PAN_MOTOR		ipwm06
#define TILT_MOTOR		ipwm07
#define HARVESTER_MOTOR ipwm08
#define MOTORSIGN 1  // Reverse the motors on one side

// Analog Inputs
#define GYROINPUT   rc_ana_in01
#define SHOOTERTACH Get_Analog_Value(rc_ana_in02)
#define PANPOT      Get_Analog_Value(rc_ana_in03)
#define TILTPOT     Get_Analog_Value(rc_ana_in04)

// Unused
#define CAM_TILT_POSITION	Get_Analog_Value(rc_ana_in04)
#define CAM_PAN_POSITION	Get_Analog_Value(rc_ana_in03)

// Digital Inputs
//#define TILTLIMIT   rc_dig_in03
//#define PANLIMITCCW rc_dig_in04
//#define PANLIMITCW  rc_dig_in05

// Interrupts
//Left Encoder (rc_dig_in01)
//Right Encoder (rc
//Other_dig_in02)

#define LEFT_B_ENCODER  rc_dig_in07   //Quad encoder B signal
#define RIGHT_B_ENCODER rc_dig_in08  //Quad encoder B signal
#define BALLREADY       rc_dig_in09  //Ball limit switch
#define PISTONUP        rc_dig_in10  //Piston Reed switch

// Field start positions
#define FIELD_LEFT   rc_dig_in11
#define FIELD_CENTER rc_dig_in12
#define FIELD_RIGHT  rc_dig_in13

#define PRESSURE     rc_dig_in06

//Autonomous selection
// rc_dig_in15 - rc_dig_in18
#define AUTOSELECT1 rc_dig_in15
#define AUTOSELECT2 rc_dig_in16
#define AUTOSELECT3 rc_dig_in17
#define AUTOSELECT4 rc_dig_in18
#define AUTOMODE ~PORTJ>>4

/************ OI Inputs ************/
// Drive joysticks & controls
#define JOY_DRIVE1A	ip1_y // Both 1 & 2 joystick driving
#define JOY_DRIVE1B	ip1_x // 1 joystick driving
#define JOY_DRIVE2	ip2_y // 2 joystick driving
#define JOY_PAN     ip3_x
#define JOY_TILT    ip3_y
#define LOWGEAR     p1_sw_top
#define HIGHGEAR    p2_sw_top
#define CONVEYOR1   p1_sw_trig   // Off
#define CONVEYOR2   p2_sw_trig   // On


#define SHOOTER_INPUT_X	ip3_x
#define SHOOTER_INPUT_Y	ip3_y
#define FIRE1           p3_sw_trig

//OI Switch box
#define SHIFT           p4_sw_trig
#define JOYSELECT       p4_sw_top
#define FIRE2           p4_sw_aux1
#define SHOOTER_POWER   p4_sw_aux2
#define FIRE_MODE1      p1_sw_aux1
#define FIRE_MODE2      p1_sw_aux2

//Switch box
#define OISWITCH1 p4_sw_aux1  // Used to choose 1 or 2 joystick drive
#define OISWITCH2 p4_sw_aux2  // unassigned
#define OISWITCH3 p4_sw_trig  // unassigned
#define OISWITCH4 p4_sw_top   // unassigned
#define OISWITCH5 p4_sw_aux1  // unassigned
#define OISWITCH6 p4_sw_aux2  // unassigned
#define OISWITCH7 p1_sw_trig  // unassigned
#define OISWITCH8 p1_sw_top   // unassigned
#define OISWITCH9 p1_sw_aux1  // unassigned
#define OISWITCH10 p1_sw_aux2 // unassigned

#define OI_KP_POT p2_x  // PID tuning
#define OI_KI_POT p2_y  // PID tuning
#define OI_KD_POT p2_wheel  // PID tuning
#define OI_POT4 p2_aux  // unassigned


#define NULLJOYSTICK 9  //This value should be greater than the Victor neutral deadspot

/*******************************************************************************
                            TYPEDEF DECLARATIONS
*******************************************************************************/


/*******************************************************************************
                           FUNCTION PROTOTYPES
*******************************************************************************/

/* These routines reside in user_routines.c */
void User_Initialization(void);
void Process_Data_From_Master_uP(void);
void Driver_Routine(void);
void Pan_By_Pot(int,int *);

/* These routines reside in user_routines_fast.c */
void InterruptHandlerLow (void);  /* DO NOT CHANGE! */
void User_Autonomous_Code(void);  /* Only in full-size FRC system. */
void Process_Data_From_Local_IO(void);

// These are elsewhere
void Update_OI(void);


#endif
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
